<?php

// $Id: shopcart.inc,v 1.4 2011/09/10 12:17:55 dhavyd Exp $
/*
 * @file
 * File contains all functions that is used in the shopcart module.
 */


/*
 * Cart functions
 * ======================================================================
 */

/**
 * Callback for menu item /shopcart.
 * Show cart content with the possibillity to update the quantity for each item.
 *
 * @return array
 */
function shopcart_cart_form() {
  $path = drupal_get_path('module', 'shopcart');
  $total_price = 0;
  $cart_items = shopcart_content_list();
  if ($cart_items) {
    foreach ($cart_items['items'] as $nid => $data) {
      // Get ID without row number
      $pos = strpos($nid, "_");
      $nid_id = substr($nid, 0, $pos);
      // Load node
      $node = node_load($nid_id);
      $node->build_mode = 'cart';
      $checkboxes[$nid] = '';
      $form[$nid]['teaser'] = array(
        '#value' => shopcart_get_node_teaser($node, $items[$nid]['params'])
      );
      $form['quantity']['quantity_' . $nid] = array(
        '#default_value' => check_plain($data['quantity']),
        '#type' => 'textfield',
        '#size' => 4,
        '#maxlength' => 4,
        '#title' => '',
        '#description' => '',
      );
      $params = shopcart_split_params($cart_items['items'][$nid]['params']);
      if (array_key_exists("price", $params) && is_numeric($params['price'])) {
        $price = $params['price'];
        $amount = $params['price'] * $data['quantity'];
      }
      else {
        $price = $data['price'];
        $amount = $data['price'] * $data['quantity'];
      }
      $form[$nid]['price'] = array('#value' => format_number(check_plain($price), 2));
      $form[$nid]['item_amount'] = array('#value' => format_number(check_plain($amount), 2));
      $form[$nid]['node_type'] = array('#type' => 'hidden', '#value' => $type);
      $form[$nid]['params_' . $nid] = array('#type' => 'hidden', '#value' => $cart_items['items'][$nid]['params']);
      $total_price += $price * $data['quantity'];
    }
    $form['checkboxes'] = array(
      '#type' => 'checkboxes',
      '#options' => $checkboxes,
    );
    $form['remove'] = array(
      '#type' => 'button',
      '#name' => 'remove_selected',
      '#value' => t('Remove selected'),
    );
    $form['submit'] = array(
      '#type' => 'button',
      '#name' => 'update_quantity',
      '#value' => t('Update'),
    );
    $form['checkout'] = array(
      '#type' => 'submit',
      '#name' => 'checkout',
      '#value' => t('Checkout'),
    );
  }
  $total_price = format_number($total_price, 2);
  $form['total_price'] = array('#value' => $total_price);

  // Placed out because this theme function also shows message for empty cart.
  $form['#theme'] = 'cart_form';
  return $form;
}

/**
 * Validate the cart form.
 * 
 * @param array $form
 * @param array $form_state
 * 
*/
function shopcart_cart_form_validate($form, $form_state) {
  $op = $form_state['clicked_button']['#name'];

  switch ($op) {
    case 'remove_selected':
      $items_was_removed = FALSE;
      foreach ($form_state['values']['checkboxes'] as $nid) {
        if ($nid) {
          $pos = strpos($nid, "_");
          $nid_id = substr($nid, 0, $pos);
          $params = $form_state['values']['params_' . $nid];
          shopcart_remove_node($nid_id, $params);
          $items_was_removed = TRUE;
        }
      }
      if ($items_was_removed) {
        drupal_set_message(t('Selected items has been removed from your cart.'));
        drupal_goto('shopcart');
      }
      break;
    case 'checkout':
    // Do update in cart. 'update_quantity' and 'checkout' uses the same code.
    case 'update_quantity':
      $cart_was_updated = FALSE;
      foreach ($form_state['values'] as $key => $quantity) {
        if (strpos($key, 'quantity_') !== FALSE) {
          $nid = drupal_substr($key, 9);
          // Check if recieved only numbers:
          if (!is_numeric($quantity)) {
            form_set_error('quantity_' . $nid, t('Quantity must be numeric.'));
          }
          else {
            $pos = strpos($nid, "_");
            $nid_id = substr($nid, 0, $pos);
            $params = $form_state['values']['params_' . $nid];
            shopcart_update_node_quantity($nid_id, $quantity, $params);
            $cart_was_updated = TRUE;
          }
        }
      }
      if ($cart_was_updated) {
        if ($op == 'update_quantity') {
          drupal_set_message(t('Quantity and total price has been updated.'));
          drupal_goto('shopcart');
        }
      }
      break;
  }
}

/**
 * Submit handler for cart form.
 * Do the redirect to checkout page.
 *
 * @param array $form
 * @param array $form_state
 */
function shopcart_cart_form_submit($form, $form_state) {
  if ($form_state['clicked_button']['#name'] == 'checkout') {
    drupal_goto('shopcart/checkout');
  }
}

/*
 * ======================================================================
 */







/*
 * Checkout Functions
 * ======================================================================
 */

/**
 * Checkout form.
 * If user has empty cart will redirect to cart page.
 *
 * @return array FAPI array that describe a checkout form.
 */
function shopcart_checkout_form() {
  $cart_items = shopcart_content_list();
  if ($cart_items) {
    $options_country = array();
    // Hämtar lista med alla länder från modulen "Addresses"
    if (function_exists('_addresses_country_get')) {
      $countries = _addresses_country_get(variable_get('addresses_country_list', array()), variable_get('addresses_top_countries', array()));
      foreach ($countries as $key => $value) {
        $options_country[$value] = t($value);
      }
    }
    else {
      $options_country[0] = t('Modulen "Addresses" är inte installerat...');
    }
    $form['cart_items'] = array(
      '#type' => 'fieldset',
      '#title' => t('Cart content'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    );
    $form['cart_items']['list'] = array(
      '#type' => 'markup',
      '#value' => check_plain(theme('checkout_cart_items', $cart_items['items'])),
    );
    // Checkout form:
    $form['firstname'] = array(
      '#type' => 'textfield',
      '#title' => t('First name'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_firstname'],
    );
    $form['lastname'] = array(
      '#type' => 'textfield',
      '#title' => t('Last name'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_lastname'],
    );
    $form['company'] = array(
      '#type' => 'textfield',
      '#title' => t('Company'),
      '#required' => FALSE,
      '#default_value' => $_SESSION['shopcart_customers_company'],
    );
    $form['address'] = array(
      '#type' => 'textfield',
      '#title' => t('Address'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_address'],
    );
    $form['zip'] = array(
      '#type' => 'textfield',
      '#title' => t('Zip'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_zip'],
    );
    $form['postal'] = array(
      '#type' => 'textfield',
      '#title' => t('Postal address'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_postal'],
    );
    $form['country'] = array(
      '#type' => 'select',
      '#title' => t('Country'),
      '#options' => $options_country,
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_country'] ? $_SESSION['shopcart_customers_country'] : $options_country['Sweden'],
    );
    $form['phone'] = array(
      '#type' => 'textfield',
      '#title' => t('Phone'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_phone'],
    );
    $form['email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_email'],
    );
    $form['message'] = array(
      '#type' => 'textarea',
      '#title' => t('Message to') . ' ' . variable_get('site_name', 'shop'),
      '#default_value' => $_SESSION['shopcart_customers_message'],
      '#attributes' => array('class' => 'form-text'),
    );
    $form['receiver_radios'] = array(
      '#type' => 'radios',
      '#title' => t('Receiver'),
      '#required' => TRUE,
      '#options' => array(t('Same as buyer'), t('Other than buyer') . ' (' . t('You can add a personal message to receiver') . ')'),
      '#default_value' => $_SESSION['shopcart_customers_receiver'] ? $_SESSION['shopcart_customers_receiver'] : '',
    );
    // Checkout receiver form:
    $form['receiver'] = array(
      '#type' => 'fieldset',
      '#title' => t(''),
      '#attributes' => array('id' => 'shopcart-checkout-receiver-fieldset'),
    );
    $form['receiver']['firstname_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('First name'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_firstname_receiver'],
    );
    $form['receiver']['lastname_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('Last name'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_lastname_receiver'],
    );
    $form['receiver']['company_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('Company'),
      '#required' => FALSE,
      '#default_value' => $_SESSION['shopcart_customers_company_receiver'],
    );
    $form['receiver']['address_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('Address'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_address_receiver'],
    );
    $form['receiver']['zip_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('Zip'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_zip_receiver'],
    );
    $form['receiver']['postal_receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('Postal address'),
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_postal_receiver'],
    );
    $form['receiver']['country_receiver'] = array(
      '#type' => 'select',
      '#title' => t('Country'),
      '#options' => $options_country,
      '#required' => TRUE,
      '#default_value' => $_SESSION['shopcart_customers_country_receiver'] ? $_SESSION['shopcart_customers_country_receiver'] : $options_country['Sweden'],
    );
    $form['receiver']['message_receiver'] = array(
      '#type' => 'textarea',
      '#title' => t('Message to receiver'),
      '#default_value' => $_SESSION['shopcart_customers_message_receiver'],
      '#attributes' => array('class' => 'form-text'),
    );
    $form['back_to_shop'] = array(
      '#type' => 'button',
      '#name' => 'back_to_shop',
      '#value' => t('Back to shop'),
    );
    $form['checkout'] = array(
      '#type' => 'submit',
      '#name' => 'order',
      '#value' => t('Send order'),
    );
    return $form;
  }
  else {
    // Cart is empty and Checkout page shouldn't be shown. Redirect to shopcart:
    drupal_goto('shopcart');
  }
}

/**
 * Validate handler for checkout form.
 * Check if the e-mail field contains a valid e-mail.
 * Chech if the phone field contains a valid telephone number
 *
 * @param array $form
 * @param array $form_state
 */
function shopcart_checkout_form_validate($form, $form_state) {
  $op = $form_state['clicked_button']['#name'];
  switch ($op) {
    case 'back_to_shop':
      drupal_goto(variable_get('shopcart_admin_shop_path', ''));
      break;
    case 'order':
      // Check email field:
      if (!valid_email_address($form_state['values']['email'])) {
        form_set_error('email', t('The email address appears to be invalid') . '.');
      }
      if (!shopcart_valid_phone($form_state['values']['phone'])) {
        form_set_error('phone', t('The phone appears to be invalid') . '.');
      }
      break;
  }
}

/**
 * Submit handler for checkout form.
 * Check if the user's e-mail already exists in the database.
 * If the user exists, the information about him/her wil be updated.
 * Do the same for the receiver.
 * Register the order in the database.
 * Register each order item in the database.
 * Check if the "Payment" plugin exists, in this case redirect the user
 * to the payment page.
 * If the "Payment" plugin doesn't exist, redirect to confirmation page.
 *
 * @param array $form
 * @param array $form_state
 */
function shopcart_checkout_form_submit($form, $form_state) {
  $op = $form_state['clicked_button']['#name'];
  switch ($op) {
    case 'order':
      // Order information
      $values = $form_state['values'];
      $email = $values['email'];
      $firstname = $values['firstname'];
      $lastname = $values['lastname'];
      $company = $values['company'];
      $address = $values['address'];
      $zip = $values['zip'];
      $postal = $values['postal'];
      $country = $values['country'];
      $phone = $values['phone'];
      $usertype = 0;
      $message = $values['message'];
      $receiver_radios = $values['receiver_radios'];
      // Kontrollerar om beställarens uppgifter finns i databasen, utifrån e-postadressen
      $sql = "SELECT * FROM {%s} WHERE email = '%s' AND usertype = %d LIMIT 0,1";
      $result_select = db_query($sql, SHOPCART_USER_TABLE_NAME, check_plain($email), check_plain($usertype));
      // Uppdaterar befintliga uppgifter i databasen
      if ($result_select->num_rows > 0) {
        $sql = "UPDATE {%s} SET firstname = '%s', lastname = '%s', company = '%s', address = '%s', zip = '%s',
          postal = '%s', country = '%s', phone = '%s' WHERE email = '%s' AND usertype = %d";
        $result = db_query($sql, SHOPCART_USER_TABLE_NAME, check_plain($firstname), check_plain($lastname), check_plain($company), check_plain($address), check_plain($zip), check_plain($postal), check_plain($country), check_plain($phone), check_plain($email), check_plain($usertype));
        $data = db_fetch_array($result_select);
        $buyerid = $data['id'];
      }
      // Lägger in uppgifter för beställaren i databasen
      else {
        $sql = "INSERT INTO {%s} (email, firstname, lastname, company, address, zip, postal, country, phone, usertype)
          VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', %d)";
        $result = db_query($sql, SHOPCART_USER_TABLE_NAME, check_plain($email), check_plain($firstname), check_plain($lastname), check_plain($company), check_plain($address), check_plain($zip), check_plain($postal), check_plain($country), check_plain($phone), check_plain($usertype));
        $buyerid = db_last_insert_id(SHOPCART_USER_TABLE_NAME, 'id');
      }
      // Om mottagaren är en annan än köparen
      if ($receiver_radios == 1) {
        $firstname_receiver = $values['firstname_receiver'];
        $lastname_receiver = $values['lastname_receiver'];
        $company_receiver = $values['company_receiver'];
        $address_receiver = $values['address_receiver'];
        $zip_receiver = $values['zip_receiver'];
        $postal_receiver = $values['postal_receiver'];
        $country_receiver = $values['country_receiver'];
        $message_receiver = $values['message_receiver'];
        $phone_receiver = '';
        $usertype_receiver = 1;
        // Kontrollerar om mottagren finns i databasen, utifrån e-postadressen
        $sql_receiver = "SELECT * FROM {%s} WHERE email = '%s' AND usertype = %d LIMIT 0,1";
        $result_select_receiver = db_query($sql_receiver, SHOPCART_USER_TABLE_NAME, check_plain($email), check_plain($usertype_receiver));
        // Uppdaterar befintliga uppgifter i databasen
        if ($result_select_receiver->num_rows > 0) {
          $sql_receiver = "UPDATE {%s} SET firstname = '%s', lastname = '%s', company = '%s', address = '%s', zip = '%s',
          postal = '%s', country = '%s' WHERE email = '%s' AND usertype = %d";
          $result_receiver = db_query($sql_receiver, SHOPCART_USER_TABLE_NAME, check_plain($firstname_receiver), check_plain($lastname_receiver), check_plain($company_receiver), check_plain($address_receiver), check_plain($zip_receiver), check_plain($postal_receiver), check_plain($country_receiver), check_plain($email), check_plain($usertype_receiver));
          $data = db_fetch_array($result_select_receiver);
          $receiverid = $data['id'];
        }
        // Lägger in uppgifter för beställaren i databasen
        else {
          $sql_receiver = "INSERT INTO {%s} (email, firstname, lastname, company, address, zip, postal, country, phone, usertype)
          VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', %d)";
          $result_receiver = db_query($sql_receiver, SHOPCART_USER_TABLE_NAME, check_plain($email), check_plain($firstname_receiver), check_plain($lastname_receiver), check_plain($company_receiver), check_plain($address_receiver), check_plain($zip_receiver), check_plain($postal_receiver), check_plain($country_receiver), check_plain($phone_receiver), check_plain($usertype_receiver));
          $receiverid = db_last_insert_id(SHOPCART_USER_TABLE_NAME, 'id');
        }
      }
      // Insert order in the database
      $datetime = date("Y-m-d H:i:s");
      $status = 0;
      $sql = "INSERT INTO {%s} (buyerid, receiverid, message, message_receiver, datetime, status)
          VALUES (%d, %d, '%s', '%s', '%s', %d)";
      $result = db_query($sql, SHOPCART_ORDERS_TABLE_NAME, check_plain($buyerid), check_plain($receiverid), check_plain($message), check_plain($message_receiver), check_plain($datetime), check_plain($status));
      $orderid = db_last_insert_id(SHOPCART_ORDERS_TABLE_NAME, 'id');
      $order_amount = 0;
      $sid = shopcart_get_sid($sid);
      $sql = "SELECT * FROM {%s} WHERE sid = '%s'";
      $result = db_query($sql, SHOPCART_TABLE_NAME, check_plain($sid));
      if ($result->num_rows > 0) {
        while ($data = db_fetch_array($result)) {
          $node = node_load($data['nid']);
          $price_field = 'field_shopcart_product_price';
          $price_field = $node->$price_field;
          $pos = strpos($data['params'], "price");
          if ($pos !== FALSE) {
            $params = shopcart_split_params($data['params']);
            $price = $params['price'];
          }
          else {
            $price = $price_field[0]['amount'];
          }
          $order_amount = $order_amount + ($price * $data['quantity']);
          // Register each product in the order
          $sql2 = "INSERT INTO {%s} (orderid, productid, price, quantity, params)
            VALUES (%d, %d, %d, %d, '%s')";
          $result2 = db_query($sql2, SHOPCART_ORDER_ITEMS_TABLE_NAME, check_plain($orderid), check_plain($data['nid']), check_plain($price), check_plain($data['quantity']), check_plain($data['params']));
        }
        $shipping_fee = variable_get('shopcart_admin_shipping_fee', FALSE);
        if ($shipping_fee) {
          $order_amount = $order_amount + $shipping_fee;
          $sql2 = "INSERT INTO {%s} (orderid, productid, price, quantity, params)
            VALUES (%d, %d, %d, %d, '%s')";
          $result2 = db_query($sql2, SHOPCART_ORDER_ITEMS_TABLE_NAME, check_plain($orderid), check_plain(0), check_plain($shipping_fee), check_plain(1), check_plain('shipping'));
        }
        // Empty the cart if the database returns TRUE
        if ($result2 === TRUE) {
          shopcart_empty_cart();
        }
      }
      $shopcart_plugins = shopcart_get_plugins();
      // If plugin "payment" is available, then it will be used
      if (in_array('payment', $shopcart_plugins)) {
        $payment_parameters = array(
          'orderid' => $orderid,
          'amount' => $order_amount,
        );
        // Call the payment function
        print shopcart_plugins_func_get_value('payment', $payment_parameters);
      }
      else {
        // Uncomment the lines below if you want to set the order as paid
//        $status = 1;
//        $sql = "UPDATE {%s} SET status = %d WHERE id = %d";
//        $result = db_query($sql, SHOPCART_ORDERS_TABLE_NAME, check_plain($status), check_plain($orderid));
        drupal_goto('shopcart/order/confirm/' . $orderid);
      }
      break;
  }
}
/*
 * ======================================================================
 */

/*
 * API Functions
 * ======================================================================
 */

/**
 * Do different actions and redirect the user to the destination page.
 *
 * @param string $op
 * @param integer $nid
 * @param integer $quantity
 * @param string $params
 */
function shopcart_actions($op, $nid, $quantity=1, $params='0') {
  switch ($op) {
    case 'add':
      shopcart_add_node($nid, $quantity, $params);
      break;
    case 'remove':
      shopcart_remove_node($nid, $params);
      break;
  }
  if (isset($_REQUEST['destination'])) {
    $destination = $_REQUEST['destination'];
  }
  else {
    $destination = variable_get('shopcart_admin_shop_path', '');
  }
  drupal_goto($destination);
}

/**
 * Adds Shopcart product to cart.
 * If product already is in cart, the quantity will be increased.
 *
 * @param integer $nid Shopcart product's ID
 * @param integer $quantity Number of Shopcart product's copy which will be added to cart.
 * @param string $params Paramenters for the given product.
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 * @return integer Return the current quantity for given node in cart.
 */
function shopcart_add_node($nid, $quantity=0, $params='', $sid='') {
  $sid = shopcart_get_sid($sid);
  $node_quantity = shopcart_get_quantity($nid, $sid, $params);
  if ($node_quantity === FALSE) {
    // Add new node
    if ($quantity > 0) {
      $sql = "INSERT INTO {%s} (sid, nid, quantity, params) VALUES ('%s', %d, %d, '%s')";
      db_query($sql, SHOPCART_TABLE_NAME, check_plain($sid), check_plain($nid), check_plain($quantity), check_plain($params));
    }
    else {
      // Do not insert node with negative quantity!
      return FALSE;
    }
  }
  else {
    // Node already in cart
    $quantity = $node_quantity + $quantity;
    if ($quantity > 0) {
      $sql = "UPDATE {%s} SET quantity = %d WHERE sid = '%s' AND nid = %d AND params = '%s'";
      db_query($sql, SHOPCART_TABLE_NAME, check_plain($quantity), check_plain($sid), check_plain($nid), check_plain($params));
    }
    else {
      // Node has negative quantity and should be removed:
      shopcart_remove_node($nid, $params, $sid);
    }
  }
  return $quantity;
}

/**
 * Removes product with given nid from cart.
 *
 * @param integer $nid Node ID.
 * @param string $params Parameters for the given product.
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 */
function shopcart_remove_node($nid, $params='', $sid='') {
  $sid = shopcart_get_sid($sid);
  $sql = "DELETE FROM {%s} WHERE sid='%s' AND nid=%d AND params='%s'";
  db_query($sql, SHOPCART_TABLE_NAME, check_plain($sid), check_plain($nid), check_plain($params));
}

/**
 * Returns session ID for current user if $sid is empty
 *
 * @param string $sid Session ID
 * @return string Returns session ID
 */
function shopcart_get_sid($sid='') {
  if (!$sid) {
    return $_COOKIE[session_name()];
  }
  else {
    return $sid;
  }
}

/**
 * Returns quantity for given node in cart.
 *
 * @param integer $nid Node ID.
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 * @param string $params Parameters for the given product.
 * @return integer Number of given product in cart.
 */
function shopcart_get_quantity($nid, $sid='', $params='') {
  $sid = shopcart_get_sid($sid);
  $sql = "SELECT quantity FROM {%s} WHERE (sid='%s' AND nid=%d AND params='%s')";
  return db_result(db_query($sql, SHOPCART_TABLE_NAME, $sid, $nid, check_plain($params)));
}

/**
 * Returns cart content in array format.
 *
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 * @return mixed Returns FALSE if cart is empty or array of product's data:
 *      ['items']
 *        key = nid_params
 *        value = array(
 *          'nid' => $data['nid'],
 *          'title' => $data['title'],
 *          'quantity' => $data['quantity'],
 *          'price' => $data['price'],
 *          'params' => $data['params'],
 *        );
 *      ['total_price']
 */
function shopcart_content_list($sid='') {
  $sid = shopcart_get_sid($sid);
  $sql = "SELECT sc.nid, n.title, sc.quantity, sc.params
          FROM {%s} sc
            LEFT JOIN {node} n USING (nid)
          WHERE sc.sid='%s'
            AND sc.quantity > 0
            AND n.status = 1";
  $result = db_query($sql, SHOPCART_TABLE_NAME, $sid);
  $nid_list = array();
  while ($data = db_fetch_array($result)) {
    $node = node_load($data['nid']);
    // Check the product price exists, if not, set the price to zero
    $price_field = 'field_shopcart_product_price';
    if ($price_field) {
      $price = $node->$price_field;
    }
    else {
      $price = 0;
    }
    // Extract product ID
    $row_id = $data['nid'] . "_" . $data['params'];
    // Split the parameters
    $params = shopcart_split_params($data['params']);
    // If the parameters contain price information, use this for the product price
    if ($params['price']) {
      $price[0]['amount'] = check_plain($params['price']);
    }
    $nid_list['items'][$row_id] = array(
        'nid' => check_plain($data['nid']),
        'title' => check_plain($data['title']),
        'quantity' => check_plain($data['quantity']),
        'price' => check_plain($price[0]['amount']),
        'params' => check_plain($data['params']),
    );
    $nid_list['total_price'] += $data['quantity'] * $price[0]['amount'];
  }
  if (count($nid_list) > 0) {
    return $nid_list;
  }
  else {
    return FALSE;
  }
}

/**
 * Returns an array after the params string is exploded.
 *
 * @param string $params Parameters for a product.
 * @return array Returns empty array if params length is to short or array with the params:
 *        key = param type
 *        value = value for the param (ex: node ID if param type is a node type)
 */
function shopcart_split_params($params) {
  $output = array();
  if (strlen($params) > 2) {
//    $params = substr($params, 0, strlen($params) - 1); // Remove the last ';' from the string
    if (($params != '0') || ($params != '')) {
      $params_exp = explode(";", $params);
      foreach ($params_exp as $key => $value) {
        $params_key_value = explode("=", $params_exp[$key]);
        $output[$params_key_value[0]] = $params_key_value[1];
      }
    }
  }
  else {
    $output = array();
  }
  return $output;
}

/**
 * Returns a string a HTML presentation of node title and its params.
 *
 * @param stdClass $node A node.
 * @param string $params Parameters for the product/node.
 * @return string DIV box with the product/node title, plus returned title and
 *                value for the params
 */
function shopcart_get_node_teaser($node, $params) {
  $params_splited = shopcart_split_params($params);
  $output = '';
  $output .= '<div class="shopcart-cart-items-title">' . $node->title . '</div>';
  if ((array_count_values($params_splited) > 0) && (!array_key_exists("price", $params_splited))) {
    foreach ($params_splited as $key => $value) {
      $result = shopcart_plugins_func_get_value($key, $value);
      $output .= '<div><span class="shopcart-cart-items-extras-title">' . $result[$key]['title'] . ': </span><span class="shopcart-cart-items-extras-value">' . $result[$key]['value'] . '</span></div>';
    }
  }
  return $output;
}

/**
 * Sets given quantity for given product.
 *
 * @param integer $nid Node ID.
 * @param integer $quantity Quantity of nodes.
 *        If positive - nodes will be added to cart.
 *        If negative - this node will be removed.
 * @param string $params Parameters for the product.
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 */
function shopcart_update_node_quantity($nid, $quantity, $params=0, $sid='') {
  $sid = shopcart_get_sid($sid);
  if ($quantity > 0) {
    $sql = "UPDATE {%s} SET quantity = %d WHERE sid = '%s' AND nid = %d AND params = '%s'";
    db_query($sql, SHOPCART_TABLE_NAME, check_plain($quantity), check_plain($sid), check_plain($nid), check_plain($params));
  }
  else {
    // Node has negative quantity or equal zero and should be removed:
    shopcart_remove_node($nid, $params, $sid);
  }
}

/**
 * Remove all cart content for given Session ID.
 *
 * @param string $sid Session ID
 *        If $sid was not specified - the current user session ID will be used.
 */
function shopcart_empty_cart($sid='') {
  $sid = shopcart_get_sid($sid);
  $sql = "DELETE FROM {%s} WHERE sid='%s'";
  db_query($sql, SHOPCART_TABLE_NAME, check_plain($sid));
}

/**
 * Do validation of phone number
 *
 * @param string $phone
 * @return boolean Returns TRUE if phone is valid. Returns FALSE if not valid.
 */
function shopcart_valid_phone($phone) {
  if (!$phone)
    return FALSE;
  // Leave only numbers
  $phone_number = preg_replace("/[^0-9]/", "", $phone);
  if (drupal_strlen($phone_number)) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Get text for order status
 *
 * @param integer $status Order status code
 * @return string Returns order status.
 */
function shopcart_get_status_text($status) {
  $status_text = array(
      0 => t('Not paid'),
      1 => t('Paid'),
      2 => t('Delivered'),
  );
  
  return $status_text[$status];
}

/**
 * Order handlers: Order confirmation.
 *                 Order error (used when payment fails.
 *
 * @param string $op Option for which action to take.
 *                   Available actions: 'confirm', 'error'
 * @param integer $orderid Order ID.
 * @return string Returns themed order or error message.
 */
function shopcart_order_handler($op, $orderid) {
  switch ($op) {
    // Order confirmation
    case 'confirm':
      // Get order
      $order = shopcart_order_actions('getorder', $orderid);
      $to = $order['buyer']['email'];
      $email_from = variable_get('site_mail', '');
      $from = '"'. mime_header_encode(variable_get('site_name', 'Drupal Site')) .'" <'. $email_from .'>';
      $params['headers'] = array(
        'From' => $from,
        );
      $params['order'] = $order;
      $params['body'] = theme('email_order_confirmation', $order);
      // Send confirmation e-mail
      $sent = drupal_mail('shopcart', 'shopcart-mail-customer-order', $to, language_default(), $params);
      if ($sent['result'] !== TRUE) {
        drupal_set_message(t('It was not possible to send your order by e-mail...'), 'error');
      }
      // Send confirmation e-mail to administrator, if it's available
      $admin_email = variable_get('shopcart_admin_email', '');
      if ($admin_email) {
        $sent = drupal_mail('shopcart', 'shopcart-mail-admin-order', $admin_email, language_default(), $params, $from);
      }
      // Title if order was successfully registred
      drupal_set_title(variable_get('shopcart_order_title_succeed', ''));
      $output = '';
      // Show order confirmation
      $output .= theme('showorder', $order);
      // Get the messages for every item category in the order
      if (isset($order['items'])) {
        foreach ($order['items'] as $key => $value) {
          $output .= theme('order_confirmation_text', variable_get('shopcart_category_' . $value['category'], ''));
        }
      }
      $output .= theme('order_confirmation_text', variable_get('shopcart_order_text_succeed', ''));
      return $output;
      break;
    // Order error, if payment fails
    case 'error':
      drupal_set_title(t('Purchase canceled'));
      $output = array(
          'orderid' => $orderid,
      );
      return theme('order_error', $output);
      break;
  }
}

/**
 * Order actions
 *
 * @param string $op Option for which action to take.
 *                   Available actions: 'showorder', 'getorder', 'editorder'
 * @param integer $orderid Order ID.
 * @return mixed Showorder: string with themed order.
 *               Getorder: array with the order.
 *               Editorder: rendered form for order edit.
 */
function shopcart_order_actions($op, $orderid) {
  // Set the breadcrumb
  $breadcrumb = array();
  $breadcrumb[] = l(t('Home'), '<front>');
  $breadcrumb[] = l(t('Admin'), 'admin');
  $breadcrumb[] = l(t('Reports'), 'admin/reports');
  $breadcrumb[] = l(t('Shopcart'), 'admin/reports/shopcart');
  drupal_set_breadcrumb($breadcrumb);

  switch ($op) {
    // Show order
    case 'showorder':
      // Get order
      $order = shopcart_order_actions('getorder', $orderid);
      return theme('showorder', $order);
      break;
    // Get order
    case 'getorder':
      // Get order
      $views_shopcart_orders = views_get_view_result('Shopcart_orders', 'default', array($orderid));
      // Get order items
      $views_order_items = views_get_view_result('Shopcart_order_items', 'default', array($orderid));
      // Stores order in variable      
      $views_orders = $views_shopcart_orders[0];
      // Build the order in an array
      $order = array(
          'id' => $views_orders->id,
          'status' => $views_orders->shopcart_orders_status,
          'items' => array(),
          // Pesonal information (buyer)
          'buyer' => array(
              'id' => $views_orders->shopcart_orders_buyerid,
              'firstname' => $views_orders->shopcart_user_shopcart_orders_firstname,
              'lastname' => $views_orders->shopcart_user_shopcart_orders_lastname,
              'company' => $views_orders->shopcart_user_shopcart_orders_company,
              'address' => $views_orders->shopcart_user_shopcart_orders_address,
              'zip' => $views_orders->shopcart_user_shopcart_orders_zip,
              'postal' => $views_orders->shopcart_user_shopcart_orders_postal,
              'country' => $views_orders->shopcart_user_shopcart_orders_country,
              'phone' => $views_orders->shopcart_user_shopcart_orders_phone,
              'email' => $views_orders->shopcart_user_shopcart_orders_email,
          ),
          // Pesonal information (receiver)
          'receiver' => array(
              'id' => $views_orders->shopcart_orders_receiverid,
              'firstname' => $views_orders->shopcart_user_shopcart_orders_1_firstname,
              'lastname' => $views_orders->shopcart_user_shopcart_orders_1_lastname,
              'company' => $views_orders->shopcart_user_shopcart_orders_1_company,
              'address' => $views_orders->shopcart_user_shopcart_orders_1_address,
              'zip' => $views_orders->shopcart_user_shopcart_orders_1_zip,
              'postal' => $views_orders->shopcart_user_shopcart_orders_1_postal,
              'country' => $views_orders->shopcart_user_shopcart_orders_1_country,
          ),
          'message' => $views_orders->shopcart_orders_message,
          'message_receiver' => $views_orders->shopcart_orders_message_receiver,
          'datetime' => $views_orders->shopcart_orders_datetime,
          'status' => $views_orders->shopcart_orders_status,
          'comment' => $views_orders->shopcart_orders_comment,
      );
      $category_str = '';
      // Get items from an order
      foreach ($views_order_items as $key => $value) {
        if ($value->node_shopcart_order_items__term_data_tid) {
          $category = $value->node_shopcart_order_items__term_data_tid;
          $category_str .= $category . ';';
        }
        $order['items'][$key] = array(
            'id' => $value->shopcart_orders_shopcart_order_items_id,
            'title' => $value->node_shopcart_order_items_title,
            'productid' => $value->node_shopcart_order_items_nid,
            'price' => $value->shopcart_order_items_price,
            'quantity' => $value->shopcart_order_items_quantity,
            'params' => $value->shopcart_order_items_params,
            'category' => $value->node_shopcart_order_items__term_data_tid,
        );
      }
      if (strlen($category_str) > 0) {
        $category_str = substr($category_str, 0, strlen($category_str) - 1);
        // Store the item categories in a SESSION 
        $_SESSION['shopcart_products_categories'] = $category_str;
      }
      return $order;
      break;

    // Edit order
    case 'editorder':
      // Get order
      $views_orders = views_get_view_result('Shopcart_orders', 'default', array($orderid));
      return drupal_get_form('shopcart_editorder_form', $views_orders[0]);
      break;
  }
}

/**
 * Get the ID of the "Shopcart categories" vocabulary.
 *
 * @return integer The ID of the vocabulary.
 */
function shopcart_get_vocabulary() {
  // Check the vocabularies associated to node type 'shopcart_product' and return the vid
  $vocabularies = taxonomy_get_vocabularies('shopcart_product');
  foreach ($vocabularies as $key => $value) {
    if ($value->name == 'Shopcart categories') {
      return $value->vid;
      break;
    }
  }
  // Check if any vocabulary has the name 'Shopcart categories' and return the vid
  $vocabularies = taxonomy_get_vocabularies();
  foreach ($vocabularies as $key => $value) {
    if ($value->name == 'Shopcart categories') {
      return $value->vid;
      break;
    }
  }
  return FALSE;
}

/**
 * Calculate the VAT that is included in the amount.
 *
 * @param integer $amount Amount to calculate VAT from.
 * @return float VAT amount.
 */
function shopcart_get_vat($amount=0) {
  $output = 0;
  $vat = variable_get('shopcart_admin_vat', FALSE);
  // Check if VAT is set and if it is between 1 and 100 procent
  if (($vat) && (($vat > 0) && ($vat <= 100))) {
    // Build division factor
    $division = ($vat / 100) + 1;
    // Calculate VAT
    $output = $amount - ($amount / $division);
    return $output;
  }
  else {
    return FALSE;
  }
}

/**
 * Load order to handle with Bulk Operations.
 *
 * @param integer $orderid The order ID.
 * @return object The order in array format.
 */
function shopcart_views_bulk_operations_order_load($orderid) {
  $order = shopcart_order_actions('getorder', $orderid);
  $order['title'] = 'Order ' . $orderid;
  return (object) $order;
}
function shopcart_views_bulk_set_order_notpaid_action(&$node, $context) {
  $orderid = $context['shopcart_orders']->id;
  $status = 0;
  $sql = "UPDATE {%s} SET status = %d WHERE id = %d";
  $result = db_query($sql, SHOPCART_ORDERS_TABLE_NAME, check_plain($status), check_plain($orderid));
  return $result;
}

function shopcart_views_bulk_set_order_paid_action(&$node, $context) {
  $orderid = $context['shopcart_orders']->id;
  $status = 1;
  $sql = "UPDATE {%s} SET status = %d WHERE id = '%s'";
  $result = db_query($sql, SHOPCART_ORDERS_TABLE_NAME, check_plain($status), check_plain($orderid));
  return $result;
}

function shopcart_views_bulk_set_order_delivered_action(&$node, $context) {
  $orderid = $context['shopcart_orders']->id;
  $status = 2;
  $sql = "UPDATE {%s} SET status = %d WHERE id = '%s'";
  $result = db_query($sql, SHOPCART_ORDERS_TABLE_NAME, check_plain($status), check_plain($orderid));
  return $result;
}


/*
 * ======================================================================
 */



/*
 * Module installation functions
 * ======================================================================
 */


/**
 * Creates the "Shopcart product" content type
 *
 * @return mixed TRUE if any field
 */
function shopcart_create_content_type() {
  $fields = array();
  // Create field_shopcart_product_price if it doesn't exist
  if (!content_fields('field_shopcart_product_price', 'shopcart_product')) {
    module_enable(array('content', 'userreference'));
    $fields['field_shopcart_product_price'] = array(
        'field_name' => 'field_shopcart_product_price',
        'type_name' => 'shopcart_product',
        'display_settings' => array(
            'label' => array(
                'format' => 'above',
                'exclude' => 0,
            ),
            'teaser' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            'full' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            4 =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            'token' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
        ),
        'widget_active' => '1',
        'type' => 'money',
        'required' => '1',
        'multiple' => '0',
        'db_storage' => '1',
        'module' => 'money',
        'active' => '1',
        'locked' => '0',
        'columns' =>
        array(
            'amount' =>
            array(
                'type' => 'numeric',
                'precision' => '10',
                'scale' => '2',
                'not null' => FALSE,
                'sortable' => TRUE,
                'views' => TRUE,
            ),
            'currency' =>
            array(
                'type' => 'varchar',
                'length' => 3,
                'not null' => FALSE,
                'sortable' => TRUE,
                'views' => TRUE,
            ),
        ),
        'precision' => '10',
        'decimals' => '2',
        'widget' =>
        array(
            'currency_select_mode' => 'name',
            'currency_display_mode' => 'a|+|c',
            'decimals_display_mode' => 'field',
            'allowed_currencies' =>
            array(
                'AFN' => 0,
                'ALL' => 0,
                'DZD' => 0,
                'ARS' => 0,
                'AWG' => 0,
                'AUD' => 0,
                'AZN' => 0,
                'BSD' => 0,
                'BHD' => 0,
                'BDT' => 0,
                'BBD' => 0,
                'BYR' => 0,
                'BZD' => 0,
                'BMD' => 0,
                'BTN' => 0,
                'BOB' => 0,
                'BAM' => 0,
                'BWP' => 0,
                'BRL' => 0,
                'GBP' => 0,
                'BND' => 0,
                'BGN' => 0,
                'BIF' => 0,
                'KHR' => 0,
                'CAD' => 0,
                'CVE' => 0,
                'KYD' => 0,
                'XOF' => 0,
                'XAF' => 0,
                'CLP' => 0,
                'CNY' => 0,
                'COP' => 0,
                'KMF' => 0,
                'CRC' => 0,
                'HRK' => 0,
                'CUP' => 0,
                'CYP' => 0,
                'CZK' => 0,
                'DKK' => 0,
                'DJF' => 0,
                'DOP' => 0,
                'XCD' => 0,
                'EGP' => 0,
                'SVC' => 0,
                'ERN' => 0,
                'EEK' => 0,
                'ETB' => 0,
                'EUR' => 0,
                'FKP' => 0,
                'FJD' => 0,
                'GMD' => 0,
                'GHC' => 0,
                'GIP' => 0,
                'XAU' => 0,
                'GTQ' => 0,
                'GGP' => 0,
                'GNF' => 0,
                'GYD' => 0,
                'HTG' => 0,
                'HNL' => 0,
                'HKD' => 0,
                'HUF' => 0,
                'ISK' => 0,
                'INR' => 0,
                'IDR' => 0,
                'IRR' => 0,
                'IQD' => 0,
                'ILS' => 0,
                'JMD' => 0,
                'JPY' => 0,
                'JOD' => 0,
                'KZT' => 0,
                'KES' => 0,
                'KRW' => 0,
                'KWD' => 0,
                'KGS' => 0,
                'LAK' => 0,
                'LVL' => 0,
                'LBP' => 0,
                'LSL' => 0,
                'LRD' => 0,
                'LYD' => 0,
                'LTL' => 0,
                'MOP' => 0,
                'MKD' => 0,
                'MGA' => 0,
                'MWK' => 0,
                'MYR' => 0,
                'MVR' => 0,
                'MTL' => 0,
                'MRO' => 0,
                'MUR' => 0,
                'MXN' => 0,
                'MDL' => 0,
                'MNT' => 0,
                'MAD' => 0,
                'MZM' => 0,
                'MMK' => 0,
                'NAD' => 0,
                'NPR' => 0,
                'ANG' => 0,
                'TRY' => 0,
                'NZD' => 0,
                'NIO' => 0,
                'NGN' => 0,
                'KPW' => 0,
                'NOK' => 0,
                'OMR' => 0,
                'XPF' => 0,
                'PKR' => 0,
                'XPD' => 0,
                'PAB' => 0,
                'PGK' => 0,
                'PYG' => 0,
                'PEN' => 0,
                'PHP' => 0,
                'XPT' => 0,
                'PLN' => 0,
                'QAR' => 0,
                'RON' => 0,
                'RUB' => 0,
                'RWF' => 0,
                'WST' => 0,
                'STD' => 0,
                'SAR' => 0,
                'SCR' => 0,
                'RSD' => 0,
                'SLL' => 0,
                'XAG' => 0,
                'SGD' => 0,
                'SKK' => 0,
                'SBD' => 0,
                'SOS' => 0,
                'ZAR' => 0,
                'LKR' => 0,
                'SHP' => 0,
                'SDD' => 0,
                'SRD' => 0,
                'SZL' => 0,
                'SEK' => 0,
                'CHF' => 0,
                'SYP' => 0,
                'TWD' => 0,
                'TZS' => 0,
                'THB' => 0,
                'TOP' => 0,
                'TTD' => 0,
                'TND' => 0,
                'USD' => 0,
                'AED' => 0,
                'UGX' => 0,
                'UAH' => 0,
                'UYU' => 0,
                'UZS' => 0,
                'VUV' => 0,
                'VEB' => 0,
                'VND' => 0,
                'YER' => 0,
                'YUM' => 0,
                'ZMK' => 0,
              'ZWD' => 0,
            ),
            'default_value' =>
            array(
                0 =>
                array(
                    'amount' => '',
                    'currency' => '',
                    '_error_element' => 'default_value_widget][field_shopcart_price][0][amount',
                ),
            ),
            'default_value_php' => NULL,
            'label' => 'Price',
            'weight' => '-5',
            'description' => '',
            'type' => 'money_widget',
            'module' => 'money',
        ),
    );
  }


  if (!content_fields('field_shopcart_short_description', 'shopcart_product')) {

    $fields['field_shopcart_short_description'] = array(
        'field_name' => 'field_shopcart_short_description',
        'type_name' => 'shopcart_product',
        'display_settings' =>
        array(
            'label' =>
            array(
                'format' => 'above',
                'exclude' => 0,
            ),
            'teaser' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            'full' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            4 =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            2 =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            3 =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
            'token' =>
            array(
                'format' => 'default',
                'exclude' => 0,
            ),
        ),
        'widget_active' => '1',
        'type' => 'text',
        'required' => '1',
        'multiple' => '0',
        'db_storage' => '1',
        'module' => 'text',
        'active' => '1',
        'locked' => '0',
        'columns' =>
        array(
            'value' =>
            array(
                'type' => 'text',
                'size' => 'big',
                'not null' => FALSE,
                'sortable' => TRUE,
                'views' => TRUE,
            ),
            'format' =>
            array(
                'type' => 'int',
                'unsigned' => TRUE,
                'not null' => FALSE,
                'views' => FALSE,
            ),
        ),
        'text_processing' => '1',
        'max_length' => '',
        'allowed_values' => '',
        'allowed_values_php' => '',
        'widget' =>
        array(
            'rows' => '5',
            'size' => 60,
            'default_value' =>
            array(
                0 =>
                array(
                    'value' => '',
                    '_error_element' => 'default_value_widget][field_shopcart_short_description][0][value',
                ),
            ),
            'default_value_php' => NULL,
            'label' => 'Short description',
            'weight' => '-3',
            'description' => '',
            'type' => 'text_textarea',
            'module' => 'text',
        ),
    );
  }

  if (!content_fields('field_shopcart_product_image', 'shopcart_product')) {

    $fields['field_shopcart_product_image'] = array(
        'field_name' => 'field_shopcart_product_image',
        'type_name' => 'shopcart_product',
        'display_settings' =>
        array(
            'label' =>
            array(
                'format' => 'above',
                'exclude' => 0,
            ),
            'teaser' =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
            'full' =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
            4 =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
            2 =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
            3 =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
            'token' =>
            array(
                'format' => 'image_plain',
                'exclude' => 0,
            ),
        ),
        'widget_active' => '1',
        'type' => 'filefield',
        'required' => '0',
        'multiple' => '0',
        'db_storage' => '1',
        'module' => 'filefield',
        'active' => '1',
        'locked' => '0',
        'columns' =>
        array(
            'fid' =>
            array(
                'type' => 'int',
                'not null' => FALSE,
                'views' => TRUE,
            ),
            'list' =>
            array(
                'type' => 'int',
                'size' => 'tiny',
                'not null' => FALSE,
                'views' => TRUE,
            ),
            'data' =>
            array(
                'type' => 'text',
                'serialize' => TRUE,
                'views' => TRUE,
            ),
        ),
        'list_field' => '',
        'list_default' => '',
        'description_field' => '',
        'widget' =>
        array(
            'file_extensions' => NULL,
            'file_path' => NULL,
            'progress_indicator' => NULL,
            'max_filesize_per_file' => NULL,
            'max_filesize_per_node' => NULL,
            'max_resolution' => NULL,
            'min_resolution' => NULL,
            'alt' => NULL,
            'custom_alt' => NULL,
            'title' => NULL,
            'custom_title' => NULL,
            'title_type' => NULL,
            'default_image' => NULL,
            'use_default_image' => NULL,
            'filefield_sources' => NULL,
            'filefield_source_attach_path' => NULL,
            'filefield_source_attach_absolute' => NULL,
            'filefield_source_attach_mode' => NULL,
            'filefield_source_autocomplete' => NULL,
            'label' => 'Product image',
            'weight' => '-4',
            'description' => '',
            'type' => 'imagefield_widget',
            'module' => 'imagefield',
        ),
    );
  }

  if (count($fields) > 0) {
    // Need to load the CCK include file where content_field_instance_create() is defined.
    module_load_include('inc', 'content', 'includes/content.crud');
    foreach ($fields as $key => $value) {
      content_field_instance_create($value, FALSE);
    }
    return TRUE;
  }
  else {
    return FALSE;
  }
}

function shopcart_create_category_vocabulary() {
  $vocabulary = array(
      'name' => 'Shopcart categories', //Human readable name of the vocabulary
      'multiple' => 0, //set 1 to allow multiple selection
      'required' => 1, //set 1 to make the terms mandatory to be selected
      'hierarchy' => 0, //set 1 to allow and create hierarchy of the terms within the vocabulary
      'relations' => 0, //set 1 to set and allow relation amongst multiple terms
      'module' => 'shopcart', //provide the module name in which the vocabulary is defined and which is calling this function
      'nodes' => array('shopcart_product' => 1), //set the node to which this vocabulary will be attached to
      'weight' => 0, //set the weight to display the vocabulary in the list
  );
  $shopcart_category_vocabulary = shopcart_get_vocabulary();


  if ($shopcart_category_vocabulary) {
    $vocabulary['vid'] = $shopcart_category_vocabulary;
  }

  taxonomy_save_vocabulary($vocabulary);

  return TRUE;
}


